"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = __importDefault(require("typescript"));
const utils_1 = require("./utils");
function default_1(program, pluginOptions) {
    return (ctx) => {
        return (sourceFile) => {
            return typescript_1.default.visitEachChild(sourceFile, visitor, ctx);
            function visitor(node) {
                if (!typescript_1.default.isEnumDeclaration(node)) {
                    return typescript_1.default.visitEachChild(node, visitor, ctx);
                }
                if (!(0, utils_1.hasModifier)(node, typescript_1.default.SyntaxKind.ConstKeyword)) {
                    return node;
                }
                if (!(0, utils_1.hasModifier)(node, typescript_1.default.SyntaxKind.ExportKeyword)) {
                    const exportedNames = (0, utils_1.getExportedNamesOfSource)(program, sourceFile);
                    if (!exportedNames.includes(node.name.text)) {
                        return node;
                    }
                }
                if (sourceFile.isDeclarationFile) {
                    return typescript_1.default.visitEachChild(node, stripConstKeyword, ctx);
                }
                return transformEnum(node);
            }
        };
    };
    function stripConstKeyword(node) {
        return node.kind === typescript_1.default.SyntaxKind.ConstKeyword ? undefined : node;
    }
    function transformEnum(node) {
        const members = node.members;
        const known = new Map();
        const properties = [];
        let lastValue = -1;
        for (const member of members) {
            if (!typescript_1.default.isIdentifier(member.name)) {
                throw new Error('The name of enum member must be an identifier');
            }
            const name = member.name.getText();
            let value;
            let valueExpr;
            if (member.initializer) {
                value = (0, utils_1.evaluate)(member.initializer, known);
                if (typeof value === 'number') {
                    valueExpr = typescript_1.default.factory.createNumericLiteral(value);
                    lastValue = value;
                }
                else if (typeof value === 'string') {
                    valueExpr = typescript_1.default.factory.createStringLiteral(value);
                    lastValue = value;
                }
                else
                    throw new Error('Unsupported member value');
            }
            else {
                if (typeof lastValue === 'string') {
                    throw new Error('Invalid enum');
                }
                value = ++lastValue;
                valueExpr = typescript_1.default.factory.createNumericLiteral(value);
            }
            const assignment = typescript_1.default.factory.createPropertyAssignment(name, valueExpr);
            typescript_1.default.setSourceMapRange(assignment, typescript_1.default.getSourceMapRange(member));
            properties.push(assignment);
            known.set(name, value);
        }
        const result = typescript_1.default.factory.createVariableStatement(node.modifiers, typescript_1.default.factory.createVariableDeclarationList([
            typescript_1.default.factory.createVariableDeclaration(node.name, undefined, undefined, typescript_1.default.factory.createAsExpression(typescript_1.default.factory.createObjectLiteralExpression(properties, true), typescript_1.default.factory.createTypeReferenceNode(typescript_1.default.factory.createIdentifier('const'), undefined))),
        ], typescript_1.default.NodeFlags.Const));
        typescript_1.default.setSourceMapRange(result, typescript_1.default.getSourceMapRange(node));
        return result;
    }
}
exports.default = default_1;
